/* 
  project "Evaluations-Platine für Heimautomatisierung"
  created 2013 by Thomas Gauweiler
  copyright: Apache License 2.0, http://www.apache.org/licenses/LICENSE-2.0.html
*/
#define SKETCH_NAME "openHAB"
#define PIN_BUTTONS A5
#define PIN_BUTTON4 3
#define PIN_RED 6
#define PIN_YELLOW 5
#define PIN_GREEN 4

#define BUTTON_1    184
#define BUTTON_2    252
#define BUTTON_3    328
#define BUTTON_12   118
#define BUTTON_13   132
#define BUTTON_23   165
#define BUTTON_123   94
#define NOBUTTON   1023

#define RED_ON LOW
#define RED_OFF HIGH
#define YELLOW_ON HIGH
#define YELLOW_OFF LOW
#define GREEN_ON HIGH
#define GREEN_OFF LOW

bool switch1_state = false;
bool switch2_state = false;
bool switch3_state = false;

void switch_all_leds_on() {
  digitalWrite(PIN_RED, RED_ON);
  digitalWrite(PIN_YELLOW, YELLOW_ON);
  digitalWrite(PIN_GREEN, GREEN_ON);
}

void switch_all_leds_off() {
  digitalWrite(PIN_RED, RED_OFF);
  digitalWrite(PIN_YELLOW, YELLOW_OFF);
  digitalWrite(PIN_GREEN, GREEN_OFF);
}

void setup_serial() {
  Serial.begin(9600);
  Serial.print(SKETCH_NAME);
  Serial.println(" is ready;");
}

void setup_leds() {
  pinMode(PIN_RED, OUTPUT);
  pinMode(PIN_YELLOW, OUTPUT);
  pinMode(PIN_GREEN, OUTPUT);
  switch_all_leds_on();
  delay(1000);
  switch_all_leds_off();
  delay(1000);
}

void setup_button() {
  pinMode(PIN_BUTTON4, INPUT);
}

void setup() {
  setup_serial();
  setup_leds();
  setup_button();
}


/******************************************/

bool is_nearly(const int requirement, const int value) {
  return (requirement - 10 < value && value < requirement + 10);
}

bool is_only_button1(const int value) {
  return (is_nearly(BUTTON_1, value));
}

bool is_only_button2(const int value) {
  return (is_nearly(BUTTON_2, value));
}

bool is_only_button3(const int value) {
  return (is_nearly(BUTTON_3, value));
}

bool is_button12(const int value) {
  return (is_nearly(BUTTON_12, value));
}

bool is_button13(const int value) {
  return (is_nearly(BUTTON_13, value));
}

bool is_button23(const int value) {
  return (is_nearly(BUTTON_23, value));
}

bool is_button123(const int value) {
  return (is_nearly(BUTTON_123, value));
}

bool is_button1(const int value) {
  return (is_only_button1(value) || is_button12(value) || is_button13(value) || is_button123(value));
}

bool is_button2(const int value) {
  return (is_only_button2(value) || is_button12(value) || is_button23(value) || is_button123(value));
}

bool is_button3(const int value) {
  return (is_only_button3(value) || is_button13(value) || is_button23(value) || is_button123(value));
}


void digital_button() {
  bool buttonState;
  buttonState = digitalRead(PIN_BUTTON4);
  if (!buttonState) {
    switch_all_leds_on();
  }
}

void print_button_state(int button_no, bool state) {
  Serial.print("item=B"); Serial.print(button_no); Serial.print(", ");
  Serial.print("value=");
  Serial.print(state?"ON":"OFF");
  Serial.println(";");
}

void prints_buttons_when_changed(int buttonsValue) {
  if (switch1_state != is_button1(buttonsValue)) {
    switch1_state = is_button1(buttonsValue);
    print_button_state(1, switch1_state);
  }
  if (switch2_state != is_button2(buttonsValue)) {
    switch2_state = is_button2(buttonsValue);
    print_button_state(2, switch2_state);
  }
  if (switch3_state != is_button3(buttonsValue)) {
    switch3_state = is_button3(buttonsValue);
    print_button_state(3, switch3_state);
  }
}

void reads_command(char *command) {
  if (Serial.available() > 0) {
    Serial.readBytes(command, 50);
  }
}

void switch_led_for_command(char *command) {
  if (!strstr(command, "led") > 0)
    return;
  if (strstr(command, "red") > 0) {
    if (strstr(command, "on") > 0) {
      digitalWrite(PIN_RED, RED_ON);
    } else {
      digitalWrite(PIN_RED, RED_OFF);
    }
  }
  if (strstr(command, "yellow") > 0) {
    if (strstr(command, "on") > 0) {
      digitalWrite(PIN_YELLOW, YELLOW_ON);
    } else {
      digitalWrite(PIN_YELLOW, YELLOW_OFF);
    }
  }
  if (strstr(command, "green") > 0) {
    if (strstr(command, "on") > 0) {
      digitalWrite(PIN_GREEN, GREEN_ON);
    } else {
      digitalWrite(PIN_GREEN, GREEN_OFF);
    }
  }
}

void loop()
{
  int buttonsValue = 0;
  char command[50];
  buttonsValue = analogRead(PIN_BUTTONS);
  prints_buttons_when_changed(buttonsValue);
  reads_command(command);
  if (strstr(command, "led") > 0) {
    switch_led_for_command(command);
  }
  command[0] = '\0';
  delay (800);
}

